// src/util.ts
var camelToKebab = (s) => s.replace(/[A-Z]/g, (x) => `-${x.toLowerCase()}`);

// src/hash.ts
import xxh32 from "xxh32";
var encoder = new TextEncoder();
var hashCode = (str) => (xxh32(encoder.encode(str)) >>> 1).toString(36).padStart(7, "_");
var hash_default = hashCode;

// src/index.ts
var tokenSymbol = Symbol("$$css.token");
var isProperty = (key) => /^\$?[0-9A-Za-z_]+$/.test(key);
var isMacroKey = (key) => key[0] === "$" && key[1] === "$";
var isAtRule = (key) => key[0] === "@";
var keyToProp = (key) => key[0] === "$" ? `--${camelToKebab(key.slice(1))}` : camelToKebab(key);
var tokenFallback = (key) => key[0] === "$" ? `var(--${camelToKebab(key.slice(1))})` : key;
var copy = (obj) => Object.assign(Object.create(Object.getPrototypeOf(obj)), obj);
var chain = (fn, next) => (...args) => fn(...args) || next(...args);
var initialMacros = Object.assign(
  /* @__PURE__ */ Object.create(null),
  { [tokenSymbol]: tokenFallback }
);
var stringify = (obj, parent, macros, outMacros) => {
  const { classBody, outsideCss } = _stringify(obj, parent, macros, outMacros);
  return classBody === "" ? outsideCss : `${parent}{${classBody}}${outsideCss}`;
};
var _stringify = (obj, parent, inMacros, outMacros) => {
  const macros = Object.create(inMacros);
  let classBody = "";
  let outsideCss = "";
  for (const key of Object.keys(obj)) {
    let value = obj[key];
    if (isMacroKey(key)) {
      if (key.length === 2) {
        macros[tokenSymbol] = chain(value, macros[tokenSymbol]);
        continue;
      }
      const macroKey = key.slice(2);
      const guard = macroKey[0].toUpperCase() !== macroKey[0];
      const macro = {
        fn: value,
        table: guard ? copy(macros) : null
      };
      macros[macroKey] = macro;
    } else if (key in macros) {
      const { fn, table } = macros[key];
      const result = _stringify(fn(value), parent, table != null ? table : copy(macros), macros);
      classBody += result.classBody;
      outsideCss += result.outsideCss;
    } else if (isProperty(key)) {
      value = value.toString();
      const macro = macros[tokenSymbol];
      const prop = keyToProp(key);
      const len = value.length | 0;
      let body = "";
      let buf = "";
      for (let i = 0; i < len; i = i + 1 | 0) {
        const c = value.charCodeAt(i);
        if (c > 32 && c !== 34 && c !== 39 && c !== 40 && c !== 41 && c !== 42 && c !== 44 && c !== 47) {
          buf += String.fromCharCode(c);
          continue;
        }
        if (buf !== "") {
          body += macro(buf, key);
          buf = "";
        }
        if (c !== 34 && c !== 39) {
          body += String.fromCharCode(c);
          continue;
        }
        buf = String.fromCharCode(c);
        i = i + 1 | 0;
        while (i < len) {
          const b = value.charCodeAt(i);
          buf += String.fromCharCode(b);
          if (b === c)
            break;
          if (b === 92) {
            i = i + 1 | 0;
            if (i >= len)
              break;
            buf += String.fromCharCode(value.charCodeAt(i));
          }
          i = i + 1 | 0;
        }
        body += macro(buf, key);
        buf = "";
      }
      if (buf !== "")
        body += macro(buf, key);
      classBody += `${prop}:${body};`;
    } else if (isAtRule(key)) {
      const body = stringify(value, parent, macros);
      if (body !== "")
        outsideCss += `${key}{${body}}`;
    } else {
      const selectors = parent.split(",").map((parentSelector) => {
        const selector = parentSelector.trim();
        return key.replace(/\\?&/g, (match) => match[0] === "\\" ? match : selector);
      }).join(",");
      outsideCss += stringify(value, selectors, macros);
    }
  }
  if (outMacros !== void 0)
    Object.assign(outMacros, macros);
  return {
    classBody,
    outsideCss
  };
};
var addToHead = (textContent) => {
  const style = document.createElement("style");
  style.textContent = textContent;
  document.head.appendChild(style);
};
var basicFilterGenerator = () => {
  const filterSet = /* @__PURE__ */ new Set();
  return (hash) => {
    if (filterSet.has(hash))
      return false;
    filterSet.add(hash);
    return true;
  };
};
var $$css = (globalMacros = {}, {
  root = ":root",
  tick = queueMicrotask || setTimeout,
  flush = addToHead,
  filter = basicFilterGenerator
} = {}) => {
  const checkFilter = filter();
  const macros = Object.create(initialMacros);
  let textContent = "";
  let flushing = false;
  const tickFlush = (str = "") => {
    textContent += str;
    if (textContent !== "" && !flushing) {
      flushing = true;
      tick(tickFlush2);
    }
  };
  const tickFlush2 = () => {
    flushing = false;
    textContent = flush(textContent) || "";
    tickFlush();
  };
  for (const obj of [globalMacros].flat())
    tickFlush(stringify(obj, root, copy(macros), macros));
  const $css = (obj, className) => stringify(obj, className, macros);
  const css = (obj, className) => {
    if (className !== void 0) {
      const result2 = $css(obj, className);
      const hash2 = hash_default(result2);
      if (checkFilter(hash2, obj))
        tickFlush(result2);
      return className;
    }
    const result = $css(obj, ".$&");
    const hash = hash_default(result);
    if (checkFilter(hash, obj))
      tickFlush(result.replace(/\.\$&/g, `.${hash}`));
    return hash;
  };
  return {
    $css,
    css
  };
};
var src_default = $$css;
export {
  src_default as default
};
